
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;


class SVGWriter {

	public SVGWriter(double w, double h)
			throws TransformerException, ParserConfigurationException {

		try {
			m_document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		}
		catch( ParserConfigurationException pce ) {
			System.out.println( "Translate: trying to write an svg file: " + pce);
			pce.printStackTrace();
			throw pce;
		}

		m_root = m_document.createElement( "svg" );
		m_root.setAttribute("xmlns", "http://www.w3.org/2000/svg");
		m_root.setAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
		m_root.setAttribute("width", format(w) + "px");
		m_root.setAttribute("height", format(h) + "px");
		
		

		try {
			m_transformer = TransformerFactory.newInstance().newTransformer();
			m_transformer.setOutputProperty( "indent", "yes" );
		}
		catch( TransformerException te ) {
			System.out.println( "Translate: trying to write an svg file: " + te);
			te.printStackTrace();
			throw te;
		}
	}

	public void addPath( String path ) {
		Element pathElement = m_document.createElement( "path" );
		pathElement.setAttribute("fill", "none");
		pathElement.setAttribute("stroke", "red");
		pathElement.setAttribute("stroke-width", "4px");
		pathElement.setAttribute("stroke-opacity", "0.40");
		pathElement.setAttribute("stroke-linejoin", "round");
		pathElement.setAttribute("stroke-linecap", "round");
		pathElement.setAttribute("d", path );

		m_root.appendChild( pathElement );
	}

	public void addImage( String filename, double x, double y, double w, double h ) {
		Element imageElement = m_document.createElement( "image" );
		imageElement.setAttribute("x", x + "px");
		imageElement.setAttribute("y", y + "px");
		imageElement.setAttribute("width", w + "px");
		imageElement.setAttribute("height", h + "px");
		imageElement.setAttribute("xlink:href", filename);
		m_root.appendChild( imageElement );
	}
	
	public synchronized void addImage( String filename, double x, double y,
			double w, double h, String reference ) {
		if(reference == null) {
			addImage( filename, x, y, w, h );
			return;
		}


		Element atag = m_document.createElement("a");
		atag.setAttribute("xlink:href", reference);
		
		//very very simple stack (max size one)
		Element prevRoot = m_root;
		m_root = atag;
		
		addImage( filename, x, y, w, h );

		m_root = prevRoot;

		m_root.appendChild( atag );
	}

	public void write(OutputStream output) throws TransformerException {
		DOMSource writerSource = new DOMSource();
		Result writerResult = new StreamResult(output);

		try {
			writerSource.setNode(m_root);
			m_transformer.transform( writerSource, writerResult );
		}
		catch( TransformerException te ) {
			System.out.println( "Translate: trying to write an svg file: " + te);
			te.printStackTrace();
			throw te;
		}
	}

	/* *** FOR EXTENSIONS *** */
	public Element createElement(String type) {
		return m_document.createElement(type);
	}
	public void addElement( Element element ) {
		m_root.appendChild( element );
	}

	private static NumberFormat m_formatter = new DecimalFormat(".00");
	public static String format(double input) {
		return m_formatter.format(input);
	}


	private Document m_document;
	private Element m_root;

	private Transformer m_transformer;
}

